summaryrefslogtreecommitdiffstats
path: root/api/web_hub.go
diff options
context:
space:
mode:
Diffstat (limited to 'api/web_hub.go')
-rw-r--r--api/web_hub.go39
1 files changed, 25 insertions, 14 deletions
diff --git a/api/web_hub.go b/api/web_hub.go
index 241ebcef0..066ae3474 100644
--- a/api/web_hub.go
+++ b/api/web_hub.go
@@ -10,21 +10,23 @@ import (
)
type Hub struct {
- connections map[*WebConn]bool
- register chan *WebConn
- unregister chan *WebConn
- broadcast chan *model.Message
- stop chan string
- invalidateUser chan string
+ connections map[*WebConn]bool
+ register chan *WebConn
+ unregister chan *WebConn
+ broadcast chan *model.Message
+ stop chan string
+ invalidateUser chan string
+ invalidateChannel chan string
}
var hub = &Hub{
- register: make(chan *WebConn),
- unregister: make(chan *WebConn),
- connections: make(map[*WebConn]bool),
- broadcast: make(chan *model.Message),
- stop: make(chan string),
- invalidateUser: make(chan string),
+ register: make(chan *WebConn),
+ unregister: make(chan *WebConn),
+ connections: make(map[*WebConn]bool),
+ broadcast: make(chan *model.Message),
+ stop: make(chan string),
+ invalidateUser: make(chan string),
+ invalidateChannel: make(chan string),
}
func PublishAndForget(message *model.Message) {
@@ -37,6 +39,10 @@ func InvalidateCacheForUser(userId string) {
hub.invalidateUser <- userId
}
+func InvalidateCacheForChannel(channelId string) {
+ hub.invalidateChannel <- channelId
+}
+
func (h *Hub) Register(webConn *WebConn) {
h.register <- webConn
}
@@ -74,6 +80,11 @@ func (h *Hub) Start() {
}
}
+ case channelId := <-h.invalidateChannel:
+ for webCon := range h.connections {
+ webCon.InvalidateCacheForChannel(channelId)
+ }
+
case msg := <-h.broadcast:
for webCon := range h.connections {
if shouldSendEvent(webCon, msg) {
@@ -136,8 +147,8 @@ func shouldSendEvent(webCon *WebConn, msg *model.Message) bool {
}
}
- // Only report events to users who are in the channel for the event
- if len(msg.ChannelId) > 0 {
+ // Only report events to users who are in the channel for the event execept deleted events
+ if len(msg.ChannelId) > 0 && msg.Action != model.ACTION_CHANNEL_DELETED {
allowed := webCon.HasPermissionsToChannel(msg.ChannelId)
if !allowed {